@@ -55,7 +55,6 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 """
 
-from __future__ import division
 
 import os
 import string
@@ -421,7 +420,7 @@ def uc_from_freqscale(scale, obs, unit='ppm'):
     scale = np.array(scale)
     size = len(scale)
 
-    if unit in ['ppm', 'hz', 'khz']:
+    if unit in ('ppm', 'hz', 'khz'):
         complex = False
 
         min = scale.min()
@@ -443,7 +442,7 @@ def uc_from_freqscale(scale, obs, unit='ppm'):
             car = (min - dx / 2.0 + (max - min) / 2.0) / 1.e3
 
     else:
-        mesg = '{} is not a supported unit.'.format(unit)
+        mesg = f'{unit} is not a supported unit.'
         raise ValueError(mesg)
 
     return unit_conversion(size, complex, sw, obs, car)
@@ -458,7 +457,7 @@ def open_towrite(filename, overwrite=False, mode='wb'):
     """
     # check if file exists and overwrite if False
     if os.path.exists(filename) and (overwrite is False):
-        raise IOError("File exists, recall with overwrite=True")
+        raise OSError("File exists, recall with overwrite=True")
 
     p, fn = os.path.split(filename)  # split into filename and path
     # create directories if needed
@@ -689,12 +688,12 @@ class data_nd:
 
                 # check for out of range indexes
                 if v >= self.shape[i]:
-                    raise IndexError("index(%s) out of range(0 <= index < %s) \
-                    in dimension %s" % (v, self.shape[i] - 1, i))
+                    raise IndexError("index({}) out of range(0 <= index < {}) \
+                    in dimension {}".format(v, self.shape[i] - 1, i))
 
                 if v <= (-1 * self.shape[i] - 1):
-                    raise IndexError("index(%s) out of range(0 <= index < %s) \
-                    in dimension %s" % (v, self.shape[i] - 1, i))
+                    raise IndexError("index({}) out of range(0 <= index < {}) \
+                    in dimension {}".format(v, self.shape[i] - 1, i))
 
                 if v < 0:
                     w = self.shape[i] + v
