@@ -55,7 +55,6 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 """
 
-from __future__ import print_function, division
 
 __developer_doc__ = """
 Agilent/Varian file format information
@@ -313,7 +312,7 @@ def read(dir=".", fid_file="fid", procpar_file="procpar", read_blockhead=False,
 
     """
     if os.path.isdir(dir) is False:
-        raise IOError("directory %s does not exist" % (dir))
+        raise OSError("directory %s does not exist" % (dir))
 
     # read in the procpar file
     pdic = read_procpar(os.path.join(dir, procpar_file))
@@ -375,7 +374,7 @@ def read_lowmem(dir=".", fid_file="fid", procpar_file="procpar",
 
     """
     if os.path.isdir(dir) is False:
-        raise IOError("directory %s does not exist" % (dir))
+        raise OSError("directory %s does not exist" % (dir))
 
     # read in the procpar file
     pdic = read_procpar(os.path.join(dir, procpar_file))
@@ -1876,7 +1875,7 @@ def uninterleave_data(data):
     # determind the output dtype
     rdt = data.dtype.name
 
-    if rdt == 'int16' or rdt == "float32":
+    if rdt in ("int16", "float32"):
         cdt = "complex64"
     elif rdt == 'int32':
         cdt = "complex128"
